@@ -13,7 +13,7 @@ Modification History:
 ******************/
 
 /************************************
- Copyright (c) 2000                              RIPE NCC
+ Copyright (c) 2000,2001,2002                         RIPE NCC
 
 All Rights Reserved
 
@@ -34,19 +34,16 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ***************************************/
 
 #define DICT_INIT
+#include "rip.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <glib.h>
 #include <string.h>
-#include <stubs.h>
-#include "ca_defs.h"
-#include "ca_dictionary.h"
-#include "ca_configFns.h"
-#include <memwrap.h>
-#include <erroutines.h>
 #include <unistd.h>
 
+int num_sources=0;
+
 /* #define DEBUG */
 
 /**********************************************
@@ -463,14 +460,20 @@ ca_readConfig(const char *configFile, values_t confVars[], int size)
  *                                            *
  * Parameters                                    *
  *    configFile -- the configuration file
-  *    confVars[] -- the array of values structures            *
-  *    size -- the number of configuration variables          *
+ *    confVars[] -- the array of values structures            *
+ *    size -- the number of configuration variables          *
  *                                             *
  * Returns                                      *
  *    Nothing -- perhaps make this return 0 on successful exit ?  *
  *                                            *
  * Note:   Should we make the name of the config file a global    *
-  *      variable ?                                *
+ *      variable ?                                *
+
+ *
+ * Using UT_calloc; no need to check that a NULL pointer is
+ * returned.
+ *
+ *
  *******************************************************************/
 {
 	FILE *confPtr;	/* Pointer to config file. */
@@ -548,6 +551,11 @@ int i;		/* A counting variable used for debugging. */
 	printf("Configuration file is: %s\n", configFile);
 #endif	/* DEBUG */
 
+    /* create an aray for the multiple source data pointers */
+    confVars[CA_UPDSOURCE].valPtr = (ca_updDbSource_t **) UT_malloc(CA_MAXSOURCES);
+    /* first entry used to check for at least one valid source found */
+    ((ca_updDbSource_t **)confVars[CA_UPDSOURCE].valPtr)[0] = NULL;
+
 	/*
 	 * Open the configuration file for reading .....
 	 */
@@ -574,7 +582,6 @@ int i;		/* A counting variable used for debugging. */
 	 * file
 	 */
 
-
 	while (!feof(confPtr)) {
 
 		/*
@@ -634,35 +641,10 @@ int i;		/* A counting variable used for debugging. */
 				/*
 				 * Store a pointer to the string that
 				 * contains the value This is not necessarily
-				 * the actual value itself. First, we must
-				 * allocate some memory.
+				 * the actual value itself. 
 				 */
 
-				/******************************************************
-				 * We now use GLib strings here.  Thus, this section 	*
-				 * is commented out.												*
-				 * We use g_string_new() to create a new GString.		*
-				 * 																	*
-				 ******************************************************/
-				 
 				confVars[location].strPtr = g_string_new(value);
-
-				/*
-				 * confVars[location].strPtr = (char *) malloc(STRLENGTH_XXL);
-				 */
-
-				/*
-				 * We check the return value of the malloc
-				 * function .....
-				 */
-	
-				/*
-				 * if (confVars[location].strPtr == NULL) {
-				 *  	fprintf(stderr, "Cannot allocate memory for confVars[location].strPtr\n");
-			    *		die;
-				 *  }
-				 * strcpy(confVars[location].strPtr, value);
-				 */
 			}
 
 			/*
@@ -692,14 +674,6 @@ int i;		/* A counting variable used for debugging. */
 
 				confVars[location].valPtr = UT_malloc(sizeof(int));
 
-				/*
-				 * We no longer need to do this.
-				 *
-				 * if (confVars[location].valPtr == NULL) {
-				 * fprintf(stderr, "Cannot allocate memory !!!\n");
-           * die;
-    	    * }	
-				 */
 				sscanf(value, "%d", (int *) confVars[location].valPtr);
 				break;
 
@@ -756,15 +730,6 @@ int i;		/* A counting variable used for debugging. */
 #endif	/* DEBUG */
 				confVars[location].valPtr = (char *) UT_malloc(STRLENGTH);
 				
-				/*
-				 * Using UT_malloc => no need to do this.
-				 *
-				 *	if (confVars[location].valPtr == NULL) {
-				 *	fprintf(stderr, "Cannot allocate memory !!!\n");
-				 *	die;
-				 *	}
-				 */
-
 				strcpy(confVars[location].valPtr, value);
 				break;
 
@@ -773,25 +738,8 @@ int i;		/* A counting variable used for debugging. */
 				puts("Data type is Boolean");
 #endif	/* DEBUG */
 
-				/*
-				 * confVars[location].valPtr = (char
-				 * *)malloc(2);
-				 */
-
 				confVars[location].valPtr = UT_malloc(sizeof(int));
 
-				/*
-				 * Using UT_malloc => we do not need to do this.
-				 *
-				 *	if (confVars[location].valPtr == NULL) {
-				 *		fprintf(stderr, "Cannot allocate memory !!!\n");
-				 *		die;
-				 *	}
-				 */
-
-				/*
-				 * strcpy(confVars[location].valPtr, value);
-				 */
 				sscanf(value, "%d", (int *) confVars[location].valPtr);
 				break;
 
@@ -803,17 +751,14 @@ int i;		/* A counting variable used for debugging. */
 				/*
 				 * The elements of the Admin-DB have already
 				 * been read in.
-				 */
-				/*
 				 * Now, split up the elements and assign them
 				 * to the
-				 */
-				/* components of the Admin-DB structure. */
-				/*
+				 * components of the Admin-DB structure. 
+				 *
 				 * First, separate the values in "value",
 				 * using ',' as a
-				 */
-				/* delimiting character.  */
+				 * delimiting character.  
+                 */
 				dbcomps = g_strsplit(value, ",", 0);
 
 #ifdef DEBUG
@@ -827,16 +772,6 @@ int i;		/* A counting variable used for debugging. */
 				 */
 				newAdminPtr = UT_calloc(1, sizeof(ca_ripadmin_t));
 
-				/*
-				 * No need to check that we actually got the memory, 
-				 * because we are using UT_calloc().
-				 *
-			    *	if (newAdminPtr == NULL) {
-			    *		fprintf(stderr, "Cannot allocate memory to new admin-db structure\n");
-			    *		die;
-			    *	}
-				 */
-
 				/*
 				 * Now, assign the elements of the dbcomps
 				 * array to the appropriate components of the
@@ -869,33 +804,9 @@ int i;		/* A counting variable used for debugging. */
 				 * long-term storage.
 				 */
 
-
 				confVars[location].valPtr = (ca_ripadmin_t *) UT_calloc(1, sizeof(ca_ripadmin_t));
 
-
-				/*
-				 * We are using UT_calloc => no need to check that we 
-				 * actually got the memory.
-				 * if (confVars[location].valPtr == NULL) {
-			    *	fprintf(stderr, "Cannot allocate memory to new admin-db structure\n");
-			    *	die;
-			    * }
-				 */
-
 				memcpy(confVars[location].valPtr, newAdminPtr, sizeof(ca_ripadmin_t));
-				/*
-				 * strcpy( ((ca_ripadmin_t
-				 * *)confVars[location].valPtr)->host,
-				 * newAdminPtr->host);
-				 * (confVars[location].valPtr)->port =
-				 * newAdminPtr->port; strcpy(
-				 * (confVars[location].valPtr)->user,
-				 * newAdminPtr->user); strcpy(
-				 * (confVars[location].valPtr)->password,
-				 * newAdminPtr->password); strcpy(
-				 * (confVars[location].valPtr)->tableName,
-				 * newAdminPtr->tableName);
-				 */
 
 				UT_free(newAdminPtr);
 
@@ -908,6 +819,7 @@ int i;		/* A counting variable used for debugging. */
 			case 17:
 				/*
 				 * Found Update_Source variable.
+                   There may be multiple instances of this
 				 */
 #ifdef DEBUG
 				printf("Found Update_Source variable !!!\n");
@@ -952,17 +864,6 @@ int i;		/* A counting variable used for debugging. */
 				 */
 				newUpdDbPtr = UT_calloc(1, sizeof(ca_database_t));
 
-				/*
-				 * Using UT_calloc; no need to check that a NULL pointer is
-				 * returned.
-				 *
-				 *
-				 *	if (newUpdDbPtr == NULL) {
-				 *		fprintf(stderr, "Cannot allocate memory to new UPD DB structure.\n");
-				 *		die;
-				 *	}
-				 */
-
 				strcpy(newUpdDbPtr->host, dbcomps[0]);
 				newUpdDbPtr->port = atoi(dbcomps[1]);
 				strcpy(newUpdDbPtr->user, dbcomps[2]);
@@ -1007,16 +908,6 @@ int i;		/* A counting variable used for debugging. */
 				 */
 				newUpdSrc = UT_calloc(1, sizeof(ca_updDbSource_t));
 
-				/*
-				 * Using calloc; no need to check if a NULL pointer is
-				 * returned.
-				 *
-				 *	if (newUpdSrc == NULL) {
-				 *		fprintf(stderr, "Cannot allocate memory to new source structure\n");
-				 *		die;
-				 *	}
-				 */
-
 #ifdef DEBUG
 				puts("Created a structure for the UPD Source variable");
 #endif	/* DEBUG */
@@ -1046,25 +937,23 @@ int i;		/* A counting variable used for debugging. */
 				 * long-term storage.
 				 */
 
-				confVars[location].valPtr = (ca_updDbSource_t *) UT_calloc(1, sizeof(ca_updDbSource_t));
+                if ( num_sources < CA_MAXSOURCES )
+                {
+				  ((ca_updDbSource_t **)confVars[location].valPtr)[num_sources] = (ca_updDbSource_t *) UT_calloc(1, sizeof(ca_updDbSource_t));
 
-				/*
-				 * No need to check that we actually got the memory, because
-				 * we are using UT_calloc().
-				 *
-			    *	if (confVars[location].valPtr == NULL) {
-			    *		fprintf(stderr, "Cannot allocate memory to new admin-db structure\n");
-			    *		die;
-			    *	}
-				 */
+				  memcpy( ((ca_updDbSource_t **)confVars[location].valPtr)[num_sources++], newUpdSrc, sizeof(ca_updDbSource_t));
+                }
+                else
+                {
+				  fprintf(stderr, "Max number of Update Sources exceeded\n");
+				  die;
+                }
 
-				memcpy(confVars[location].valPtr, newUpdSrc, sizeof(ca_updDbSource_t));
-
-				/* No longer needed. */
-				UT_free(newUpdSrc);
+                /* No longer needed. */
+                UT_free(newUpdSrc);
 
 #ifdef DEBUG
-				printf("UPD-Source/DB-details/user: %s\n", (((ca_updDbSource_t *) confVars[location].valPtr)->updDb).user);
+				printf("UPD-Source/DB-details/user: %s\n", (((ca_updDbSource_t *) (confVars[location].valPtr)[num_sources-1])->updDb).user);
 #endif	/* DEBUG */
 
 				break;
@@ -1883,7 +1772,6 @@ ca_srchandle2Strelement(ca_SrcHdl_t * ah, int srcAttrib)
  *******************************************************************/
 {
 	char *myStr;
-	void ca_malloc(char *, int);
 
 	if (ah == NULL) {
 		fprintf(stderr, "ca_srchandle2Strelement(): Cannot dereference NULL pointer\n");
@@ -1942,7 +1830,7 @@ ca_srchandle2Strelement(ca_SrcHdl_t * ah, int srcAttrib)
 
 	default:
 		puts("Cannot find this source attribute");
-		break;
+                myStr = NULL;
 	}
 	pthread_mutex_unlock(&Lock);
 
@@ -2011,7 +1899,6 @@ ca_srchandle2Intelement(ca_SrcHdl_t * ah, int srcAttrib)
 	default:
 		fprintf(stderr, "Could not find source-attribute %d\n", srcAttrib);
 		die;
-		break;
 	}
 
 	pthread_mutex_unlock(&Lock);
@@ -2042,7 +1929,6 @@ ca_get_adminStrElement(int symbol, int adminAttrib)
  *******************************************************************/
 {
 	char *myStr;
-	void ca_malloc(char *, int);
 
 	/*
 	 * Make sure that we are calling the correct function.
@@ -2077,7 +1963,6 @@ ca_get_adminStrElement(int symbol, int adminAttrib)
 		default:
 			puts("Cannot find this admin attribute");
 			die;
-			break;
 		}
 		pthread_mutex_unlock(&Lock);
 
@@ -2103,40 +1988,12 @@ ca_get_adminIntElement(int symbol, int adminAttrib)
 	default:
 		puts("Cannot find this admin attribute");
 		die;
-		break;
 	}
 	pthread_mutex_unlock(&Lock);
 
 	return (myInt);
 }
 
-void
-ca_malloc(char *someStr, int memSize)
-/*******************************************************************
-  * ca_malloc                                      *
-  *  -- a function that allocates memory for a string          *
-  *                                            *
- * Parameters                                    *
-  * --someStr  - the string that is to be created              *
-  *   memSize- required amount of memory in bytes              *
-  *                                            *
-  * Returns                                      *
-  * -- nothing; it assigns the allocated memory to the pointer     *
- *   that was passed to it.                            *
-  *                                            *
-  *******************************************************************/
-{
-	someStr = malloc(memSize);
-
-	/*
-	 * Check that we actually did get the memory ....
-	 */
-	if (someStr == NULL) {
-		fprintf(stderr, "ca_malloc(): cannot allocate memory !!!\n");
-		exit(8);
-	}
-}
-
 void
 ca_set_boolean(int symbol)
 {
@@ -2563,7 +2420,7 @@ ca_set_dirlist(int symbol)
 
 	default:
 		fprintf(stderr, "Error: Unknown scope: %d\n", dictionary[symbol].varScope);
-		break;
+		die;
 	}
 
 
@@ -2627,6 +2484,11 @@ ca_set_dirlist(int symbol)
 }
 
 
+#if 0
+
+XXX: Unused function?  Removed because it causes warnings due to gets() 
+     invokation.  - Shane
+
 void
 ca_set_string(int symbol)
 {
@@ -2715,7 +2577,7 @@ ca_set_string(int symbol)
 
 	default:
 		fprintf(stderr, "Error: Unknown scope: %d\n", dictionary[symbol].varScope);
-		break;
+		die;
 	}
 
 
@@ -2782,6 +2644,7 @@ g_string_assign(hereValues[symbol].strPtr, newString);
 	 */
 
 }
+#endif /* 0 */
 
 
 int
@@ -3225,34 +3088,39 @@ ca_getType(char *confVar, dict_t woordenboek[], int size)
 }
 
 
-ca_updDbSource_t *
+ca_updDbSource_t **
 ca_get_UpdSourceHandle(int symbol)
 /*******************************************************************
- *ca_get_UpdSourceHandle                              *
- * -- returns the handle to the Update source                *
-  *                                            *
-  * Parameters                                    *
-  *  -- none; there is only one Update Source in the Configuration  *
-  *     file because a single DBupdate process cannot update more   *
- *     than one source.                              *
- *                                            *
- * Returns                                      *
-  *  -- a pointer to the Update Source structure (type           *
- *     ca_updDbSource_t) or NULL.                        *
- *                                            *
+ * ca_get_UpdSourceHandle                                          *
+ * -- returns the handle to the Update source                      *
+ *                                                                 *
+ * Parameters                                                      *
+ *  -- none; there is only one Update Source in the Configuration  *
+ *     file because a single DBupdate process cannot update more   *
+ *     than one source.                                            *
+ *     There are now multiple Update Sources in the Configuration  *
+ *     file. This was a self imposed limit that is no longer!!     *
+ *                                                                 *
+ * Returns                                                         *
+ *  -- a pointer to an array of the Update Source structure (type  *
+ *     ca_updDbSource_t) or NULL.                                  *
+ *                                                                 *
  *******************************************************************/
 {
-	ca_updDbSource_t *myUpdSourcePtr;
+	ca_updDbSource_t **myUpdSourcePtr;
 
 	/*
 	 * Make sure that we are calling the correct function.
 	 */
-	if (strcmp(dictionary[symbol].varType, "CA_UPDSOURCE") != 0) {
+	if (strcmp(dictionary[symbol].varType, "CA_UPDSOURCE") != 0) 
+    {
 		fprintf(stderr, "Error: unexpected variable type.\n");
 		die;
 	}
-	else {
-		myUpdSourcePtr = (ca_updDbSource_t *) confVars[symbol].valPtr;
+	else
+    {
+        /* get a pointer to the array of source data pointers */
+		myUpdSourcePtr = (ca_updDbSource_t **) confVars[symbol].valPtr;
 	}
 	return (myUpdSourcePtr);
 }
@@ -3321,7 +3189,7 @@ ca_UpdSrcHandle2StrElement(ca_updDbSource_t * uh, int srcAttrib)
 
 	default:
 		puts("Cannot find this Update source attribute");
-		break;
+		myStr = NULL;
 	}
 
 	return (myStr);
@@ -3375,7 +3243,6 @@ ca_UpdSrcHandle2IntElement(ca_updDbSource_t * uh, int srcAttrib)
 	default:
 		fprintf(stderr, "Could not find source-attribute %d\n", srcAttrib);
 		die;
-		break;
 	}
 
 	return (myInt);
