@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.9 $
+  $Revision: 1.16 $
 
   Status: NOT REVUED, NOT TESTED
 
@@ -9,7 +9,7 @@
   Modification History:
         andrei (10/04/2000) Created.
   ******************/ /******************
-  Copyright (c) 2000                              RIPE NCC
+  Copyright (c) 2000,2001,2002                    RIPE NCC
  
   All Rights Reserved
   
@@ -28,6 +28,9 @@
   AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ***************************************/
+
+#include "rip.h"
+
 #include <glib.h>
 #include <stdio.h>
 #include <strings.h>
@@ -36,10 +39,6 @@
 #include <ctype.h>
 #include <unistd.h>
 
-#include "nh.h"
-#include "memwrap.h"
-#include "stubs.h"
-
 /*+ String sizes +*/
 #define STR_S   63
 #define STR_M   255
@@ -119,22 +118,30 @@ char *NH_convert(nic_handle_t *nh_ptr)
 * Returns:                                                  *
 * >0 - success                                              *
 *  0 - AUTO NIC                                             *
-* -1  - error (not defined and processed yet)               *
+* -1  - error                                               *
 *                                                           *
 ************************************************************/
-int NH_parse(char *nic, nic_handle_t **nh_ptr_ptr)
+int NH_parse(const char *nic, nic_handle_t **nh_ptr_ptr)
 {
-char *ptr;
+const char *ptr;
 int res = 1;
 nic_handle_t *nh_ptr;
 
-     if(!(nh_ptr=calloc(1, sizeof(nic_handle_t)))) die;
+     *nh_ptr_ptr=NULL;
+     if(!(nh_ptr=UT_calloc(1, sizeof(nic_handle_t)))) die;
      
      ptr=nic;	
 
      /* extract space */
      while(isalpha((int)*ptr))ptr++;
-     if(!(nh_ptr->space=malloc(ptr-nic+1))) die;
+
+     /* sanity check - space <=4 characters */
+     if((ptr-nic)>MAX_NH_SPACE_LENGTH) {
+       UT_free(nh_ptr);
+       return(-1);
+     }
+
+     if(!(nh_ptr->space=UT_malloc(ptr-nic+1))) die;
      strncpy(nh_ptr->space, nic, ptr-nic); *(nh_ptr->space+(ptr-nic))='\0';
 
      /* If there are no digits, then this is no nic-hdl */
@@ -153,15 +160,32 @@ nic_handle_t *nh_ptr;
        } else {
          nic=ptr;
 	 /* convert digits (if any) and store first invalid characted in ptr */
-         nh_ptr->nic_id=(int)strtol(nic, &ptr, 10);
-	 /* Check if there were any digits at all */
-	 if(ptr == nic) nh_ptr->nic_id=NULL_NIC_ID;
+         if(isdigit((int)*nic)){
+             nh_ptr->nic_id=(int)strtol(nic, (char **)&ptr, 10);
+         } else {    
+	     /* There were no digits at all */
+	     nh_ptr->nic_id=NULL_NIC_ID;
+         }    
+       }
+       /* sanity check - if the ID does not exeed NH_MAX_ID */
+       if(nh_ptr->nic_id > MAX_NIC_ID) {
+         if(nh_ptr) {
+           UT_free(nh_ptr->space);UT_free(nh_ptr);
+         }
+         return(-1);
        }
        /* check if there is any suffix */
        if (*ptr == '\0') nh_ptr->source=NULL;
        /* Copy suffix into source */
        else {
-	 if(!(nh_ptr->source=malloc(strlen(ptr)+1))) die;
+         /* sanity check - suffix does not exceed the length */
+         if(strlen(ptr)>MAX_NH_SOURCE_LENGTH) {
+             if(nh_ptr){
+               UT_free(nh_ptr->space);UT_free(nh_ptr);
+             }
+             return(-1);
+         }
+	 if(!(nh_ptr->source=UT_malloc(strlen(ptr)+1))) die;
 	 strcpy(nh_ptr->source, ptr);
        }
      } 
@@ -347,9 +371,9 @@ long nic_id=nh_ptr->nic_id;
 void free_nh(nic_handle_t *nh_ptr)
 {
  if(nh_ptr){
-   if(nh_ptr->space)free(nh_ptr->space);
-   if(nh_ptr->source)free(nh_ptr->source);
-   free(nh_ptr);
+   if(nh_ptr->space) UT_free(nh_ptr->space);
+   if(nh_ptr->source) UT_free(nh_ptr->source);
+   UT_free(nh_ptr);
  }
 }
 
@@ -378,10 +402,7 @@ GString *query;
 long range_id=0;
 int sql_err;
 
- if ((query = g_string_sized_new(STR_L)) == NULL){ 
-  fprintf(stderr, "E: cannot allocate gstring\n"); 
-  return(-1); 
- }
+ query = g_string_sized_new(STR_L);
  
 /* Define row numbers in the result of the query */
 #define RANGE_ID 0
@@ -423,17 +444,17 @@ int sql_err;
    sql_str = SQ_get_column_string(sql_result, sql_row, RANGE_ID);
    if (sql_str != NULL) {
      range_id = atol(sql_str);
-     free(sql_str);
+     UT_free(sql_str);
    }
    sql_str = SQ_get_column_string(sql_result, sql_row, RANGE_START);
    if (sql_str != NULL) {
      prange->start = atoi(sql_str);
-     free(sql_str);
+     UT_free(sql_str);
    }
    sql_str = SQ_get_column_string(sql_result, sql_row, RANGE_END);
    if (sql_str != NULL) {
      prange->end = atoi(sql_str);
-     free(sql_str);
+     UT_free(sql_str);
    }
 
 /* We must process all the rows of the result */
@@ -477,11 +498,7 @@ long prev_range_id, next_range_id;
 int num;
 int sql_err;
 
-/* Allocate memory */
- if ((query = g_string_sized_new(STR_L)) == NULL){ 
-  fprintf(stderr, "E: cannot allocate gstring\n"); 
-  return(-1); 
- }
+ query = g_string_sized_new(STR_L);
 
 /* Do range check */
  if (( p_newrange->end > MAX_RANGE ) || ( p_newrange->start < MIN_RANGE )) return(-1);
@@ -641,11 +658,7 @@ GString *query;
 int sql_err, num;
 long range_id;
 
- /* Allocate memory */
- if ((query = g_string_sized_new(STR_L)) == NULL){ 
-  fprintf(stderr, "E: cannot allocate gstring\n"); 
-  return(-1); 
- }
+ query = g_string_sized_new(STR_L);
  
  /* get the next range_id */
  /* XXX we cannot use autoincrement with MyISAM tables */
@@ -695,11 +708,7 @@ int NH_comrol(SQ_connection_t *sql_connection, int thread_ins, int thread_del)
 GString *query;
 int sql_err;
 
- /* Allocate memory */
- if ((query = g_string_sized_new(STR_L)) == NULL){ 
-  fprintf(stderr, "E: cannot allocate gstring\n"); 
-  return(-1); 
- }
+ query = g_string_sized_new(STR_L);
  
   g_string_sprintf(query, "DELETE FROM nic_hdl "
 			  "WHERE thread_id=%d ",
