@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.61 $
+  $Revision: 1.63 $
 
   Protocol whois module (pw).  Whois protocol.
 
@@ -217,6 +217,9 @@ void PW_process_qc(Query_environ *qe,
     break;
   case QC_NOKEY:
     /* no key (this is OK for some operational stuff, like -k) */
+    /* we still need an extra newline to hand control back to the
+       client in the "-k" scenerio */
+    SK_cd_puts(&(qe->condat), "\n");
     break;       
   case QC_EMPTY:
     /* The user didn't specify a key, so
@@ -300,7 +303,9 @@ void PW_process_qc(Query_environ *qe,
 
 
       /* QI will decrement the credit counters */
+      PW_record_query_start();
       err = QI_execute(qitem->data, qis, qe, acc_credit, acl_eip );      
+      PW_record_query_end();
       if( !NOERR(err) ) { 
 	if( err == QI_CANTDB ) {
 	  SK_cd_puts(&(qe->condat), "% WARNING: Failed to make connection to ");
@@ -512,9 +517,7 @@ void PW_interact(int sock) {
     }
     else {
       /************ ACTUAL PROCESSING IS HERE ***********/
-      PW_record_query_start();
       PW_process_qc(qe, qc, &acc_credit, &acl_eip);
-      PW_record_query_end();
 
       if( qc->query_type == QC_REAL ) {
 	copy_credit.queries ++;
