@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.6 $
+  $Revision: 1.8 $
 
   Socket module - cd_socket.c - basic read/write socket routines defined
                                 in terms of connection data structures
@@ -69,6 +69,8 @@ void SK_cd_make(sk_conn_st *condat, int  sock, unsigned timeout)
   condat->eIP =  condat->rIP;
 
   condat->rd_timeout.tv_sec = timeout;
+
+  condat->rd_buf_len = 0;
 }
 
 
@@ -142,6 +144,169 @@ int SK_cd_puts(sk_conn_st *condat, const char *str)
   return res;
 } /* SK_cd_puts() */
 
+/* fill the input buffer as much as possible */
+static int
+sk_fill_rd_buf (sk_conn_st *condat)
+{
+    fd_set rset;
+    struct timeval *ptm;
+    int select_ret;
+    int rd_buf_free;
+    int read_ret;
+    
+    ptm = &(condat->rd_timeout);
+    /* if timeout is 0, do blocking I/O - bogus, but that's how it is */
+    if ((ptm->tv_sec == 0) && (ptm->tv_usec == 0)) {
+        ptm = NULL;
+    }
+    FD_ZERO(&rset);
+    FD_SET(condat->sock, &rset);
+
+    /* wait for input to become available */
+    select_ret = select(condat->sock + 1, &rset, NULL, NULL, ptm);
+    if (select_ret < 0) {
+        /* unfortunate, but bad things happen to good sockets - SK */
+        ER_perror(FAC_SK, SK_SELECT, "(%d) %s", errno, strerror(errno));
+        return -1;
+    }
+    if (select_ret == 0) {      
+        condat->rtc |= SK_TIMEOUT;
+        return -1;
+    }
+
+    /* calculate maximum amount to read */
+    rd_buf_free = sizeof(condat->rd_buf) - (condat->rd_buf_len);
+
+    /* read up to that much, if available */
+    read_ret = read(condat->sock, 
+                    condat->rd_buf + condat->rd_buf_len, 
+                    rd_buf_free);
+    if (read_ret <= 0) {
+	condat->rtc |= SK_DISCONNECT;
+        return -1;
+    }
+    condat->rd_buf_len += read_ret;
+
+    /* return the fact that we got data */
+    return read_ret;
+}
+
+#undef TEST_SK_FILL_RD_BUF
+
+/* test sk_fill_rd_buf() function - full path coverage */
+#ifdef TEST_SK_FILL_RD_BUF
+#include <unistd.h>
+#include <stdio.h>
+#include <errno.h>
+#include <string.h>
+
+int 
+main ()
+{
+    sk_conn_st conn;
+    int pfd[2];
+    int tmp_fd;
+    int bogus_fd;
+    int i;
+    int buf[INPUT_BUF_LEN+INPUT_BUF_LEN+100];
+    int fill_buf_ret;
+
+    /* test with bogus file descriptor and 0 timeout value */
+    tmp_fd = open("/dev/null", O_RDONLY);
+    if (tmp_fd == -1) {
+        fprintf(stderr, "Error %d creating bogus file descriptor; %s\n", 
+                errno, strerror(errno));
+        exit(1);
+    }
+    bogus_fd = tmp_fd;
+    close(tmp_fd);
+    SK_cd_make(&conn, bogus_fd, 0);
+    if (sk_fill_rd_buf(&conn) != -1) {
+        fprintf(stderr, "Filling buffer with bogus file descriptor worked\n");
+        exit(1);
+    }
+
+    /* create a pipe for further testing */
+    if (pipe(pfd) != 0) {
+        fprintf(stderr, "Error %d creating pipe; %s\n", errno, strerror(errno));
+        exit(1);
+    }
+    /* real file descriptor and non-0 timeout value */
+    SK_cd_make(&conn, pfd[0], 1);
+
+    /* test timeout */
+    if (sk_fill_rd_buf(&conn) != -1) {
+        fprintf(stderr, "Buffer filling failed to timeout properly\n");
+        exit(1);
+    }
+    if (conn.rtc != SK_TIMEOUT) {
+        fprintf(stderr, "Timeout didn't set flag properly\n");
+        exit(1);
+    }
+
+    /* test EOF input */
+    close(pfd[1]);
+    conn.rtc = 0;
+    if (sk_fill_rd_buf(&conn) != -1) {
+        fprintf(stderr, "Buffer filling failed to timeout properly\n");
+        exit(1);
+    }
+    if (conn.rtc != SK_DISCONNECT) {
+        fprintf(stderr, "Disconnect didn't set flag properly\n");
+        exit(1);
+    }
+
+    /* test empty buffer reading various values */
+    {
+        int lengths[] = 
+            { 1, 100, INPUT_BUF_LEN-1, INPUT_BUF_LEN, 
+              INPUT_BUF_LEN+1, INPUT_BUF_LEN+100 };
+        for (i=0; i<sizeof(lengths)/sizeof(lengths[0]); i++) {
+            close(pfd[0]);
+            close(pfd[1]);
+            if (pipe(pfd) != 0) {
+                fprintf(stderr, "Error %d creating pipe; %s\n", 
+                        errno, strerror(errno));
+                exit(1);
+            }
+            SK_cd_make(&conn, pfd[0], 0);
+            memset(buf, i+1, lengths[i]);
+            if (write(pfd[1], buf, lengths[i]) != lengths[i]) {
+                fprintf(stderr, "Error %d writing to pipe; %s\n",
+                        errno, strerror(errno));
+                exit(1);
+            }
+            fill_buf_ret = sk_fill_rd_buf(&conn);
+            if (fill_buf_ret == -1) {
+                fprintf(stderr, "Error filling buffer\n");
+                exit(1);
+            }
+            if (lengths[i] < INPUT_BUF_LEN) {
+                if (fill_buf_ret != lengths[i]) {
+                    fprintf(stderr, "Error filling buffer\n");
+                    exit(1);
+                }
+                if (memcmp(buf, conn.rd_buf, lengths[i]) != 0) {
+                    fprintf(stderr, "Error with buffer contents\n");
+                    exit(1);
+                }
+            } else {
+                if (fill_buf_ret != INPUT_BUF_LEN) {
+                    fprintf(stderr, "Error filling buffer\n");
+                    exit(1);
+                }
+                if (memcmp(buf, conn.rd_buf, INPUT_BUF_LEN) != 0) {
+                    fprintf(stderr, "Error with buffer contents\n");
+                    exit(1);
+                }
+            }
+        }
+    }
+
+    return 0;
+}
+#endif 
+
 /* SK_cd_gets() */
 /*++++++++++++++++++++++++++++++++++++++
 
@@ -178,51 +343,69 @@ int SK_cd_puts(sk_conn_st *condat, const char *str)
   ++++++++++++++++++++++++++++++++++++++*/
 int SK_cd_gets(sk_conn_st *condat, char *str, size_t count) 
 {
-  fd_set rset;
-  struct timeval *ptm = & condat->rd_timeout;
-  int readcount = 0;
-    
-  memset( str, 0, count);
-  /* leave space for \0 */
-  count--;
+    char *eol;
+    int line_len;
+    int amt_to_copy;
 
-  FD_ZERO( &rset );
-  FD_SET( condat->sock, &rset );
+    /* leave space for terminating '\0' */
+    count--;
 
-  if( ptm->tv_sec == 0 && ptm->tv_usec == 0) { /* if timeout undefined, 
-						  do blocking I/O */
-    ptm = NULL;
-  }
+    /* we must always return a NUL-terminated string */
+    str[0] = '\0';
 
-  do {
-    char buf[2];
-    int sel = select( (condat->sock)+1, &rset, NULL, NULL, ptm);
+    /* track current line length */
+    line_len = 0;
 
-    if (sel < 0) {
-      /* unfortunate, but bad things happen to good sockets - SK */
-      ER_perror(FAC_SK, SK_SELECT, "(%d) %s", errno, strerror(errno));
-      break;
-    }
-      
-    if( sel == 0 ) {      
-      condat->rtc |= SK_TIMEOUT;
-      break;
+    /* get input if none available */
+    if (condat->rd_buf_len <= 0) {
+        if (sk_fill_rd_buf(condat) == -1) {
+            return line_len;
+        }
     }
 
-    else { 
-      if(read( condat->sock, buf, 1) == 0 ) {
-	condat->rtc |= SK_DISCONNECT;
-	break;
-      }
-      str[readcount] = buf[0];
-      readcount++;
-      if( buf[0] == '\n' ) {
-	break;
-      }
-    } 
-  } while( readcount < count );
-	 
-  return readcount;
+    for (;;) {
+        /* we should always have something in the buffer here */
+        dieif(condat->rd_buf_len <= 0);
+
+        /* check for end of line */
+        eol = memchr(condat->rd_buf, '\n', condat->rd_buf_len);
+
+        /* figure out how much of the buffer to copy */
+        if (eol == NULL) {
+            /* no newline -> copy all available, if room */
+            amt_to_copy = (count - line_len);
+            if (amt_to_copy > condat->rd_buf_len) {
+                amt_to_copy = condat->rd_buf_len;
+            }
+        } else {
+            /* newline -> copy line if room */
+            amt_to_copy = eol - condat->rd_buf + 1;
+            if (amt_to_copy > (count - line_len)) {
+                amt_to_copy = count - line_len;
+            }
+        }
+
+        /* copy data to string */
+        memcpy(str + line_len, condat->rd_buf, amt_to_copy);
+        line_len += amt_to_copy;
+        str[line_len] = '\0';
+
+        /* remove data from input buffer */
+        memmove(condat->rd_buf, 
+                condat->rd_buf + amt_to_copy, 
+                condat->rd_buf_len - amt_to_copy);
+        condat->rd_buf_len -= amt_to_copy;
+
+        /* if we got a newline or our buffer is full, exit */
+        if ((eol != NULL) || (line_len >= count)) {
+            return line_len;
+        }
+
+        /* otherwise get more input */
+        if (sk_fill_rd_buf(condat) == -1) {
+            return line_len;
+        }
+    }
 
 } /* SK_cd_gets() */
 
@@ -287,7 +470,7 @@ int SK_cd_printf(sk_conn_st *condat, char *txt, ...)
     newbuf = (char *)UT_malloc(len+1);
     
     va_start(ap, txt);
-    vsnprintf(newbuf, len, txt, ap);
+    vsnprintf(newbuf, len+1, txt, ap);
     va_end(ap);   
     
     finalbuf = newbuf;
@@ -304,3 +487,4 @@ int SK_cd_printf(sk_conn_st *condat, char *txt, ...)
 
   return len;
 } /* SK_cd_printf() */
+
